package com.nisum.offertransactionservice.controller;

import com.nisum.offertransactionservice.client.ClientService;
import com.nisum.offertransactionservice.dao.EndOfSaleRepo;
import com.nisum.offertransactionservice.dao.OfferLookupRepo;
import com.nisum.offertransactionservice.dao.OfferMetaDataRepo;
import com.nisum.offertransactionservice.dto.*;
import com.nisum.offertransactionservice.model.EndOfSaleEntity;
import com.nisum.offertransactionservice.model.OfferLookup;
import com.nisum.offertransactionservice.model.OfferMeta;
import com.nisum.offertransactionservice.service.OfferCallingPEService;
import org.junit.jupiter.api.Test;
import org.mockito.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.annotation.Description;
import reactor.core.publisher.Mono;

import java.io.IOException;
import java.time.LocalDateTime;
import java.util.*;

import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.*;

@SpringBootTest
class OfferTransactionControllerTest {

    @InjectMocks
    @Autowired
    private OfferTransactionController offerTransactionController;

    @Mock
    private OfferLookupRepo offerLookupRepo;

    @Mock
    private OfferMetaDataRepo offerMetaDataRepo;

    @Mock
    private EndOfSaleRepo endOfSaleRepo;

    @Mock
    private ClientService clientService;

    private String HH_ID_FOR_END_OF_SALE = "id01";

    private String TRANSACTION_ID_FOR_END_OF_SALE = "tx01";

    private String UU_ID_FOR_END_OF_TRANSACTION = "id01";


    @Mock
    private OfferCallingPEService offerCallingPEService;

    @Description("Component test for getOfferTransactionResponse method")
    @Test
    void getOfferTransactionResponse() throws InterruptedException {
        when(offerLookupRepo.findByHhId(ArgumentMatchers.any())).thenReturn(getOffers());
        when(clientService.getPeResponseFlux(Mockito.any(PERequest.class))).thenReturn(getPeResponse());
        when(offerCallingPEService.getOfferMetaDto(ArgumentMatchers.any())).thenReturn(getOfferLookupDto());
        Mockito.doReturn(Optional.of(getOfferMeta())).when(offerMetaDataRepo).findByOfferIdAndOfferStatus(Mockito.anyLong(), Mockito.anyString());
        OfferTransactionResponse offerTransactionResponse = offerTransactionController.getOfferTransactionResponse(getOfferTransactionRequest());
        assertEquals(Objects.nonNull(offerTransactionResponse.getTransactionId()),true);
    }

    @Description("Component test for endOfSale method")
    @Test
    void endOfSale() throws IOException {
        when(endOfSaleRepo.save(Mockito.any(EndOfSaleEntity.class))).thenReturn(new EndOfSaleEntity());
        assertEquals(Objects.nonNull(offerTransactionController.endOfSale(buildEndOfSaleReq())),true);
        }

    @Description("Component test for endOfTransaction method")
    @Test
    void endOfTransaction() {
        when(clientService.getStringMono(Mockito.anyString())).thenReturn(getMonoOfString());
        assertEquals(Objects.nonNull(offerTransactionController.endOfTransaction(UU_ID_FOR_END_OF_TRANSACTION)),true);
    }

    private List<OfferLookup> getOffers() {
        OfferLookup offerLookup1 = new OfferLookup(1L, "12", 234L, "02", "AND", "BOGO");
        OfferLookup offerLookup2 = new OfferLookup(2L, "13", 234L, "02", "OR", "BXGY");
     List<OfferLookup> offerLookups = new ArrayList<>();
        offerLookups.add(offerLookup1);
        offerLookups.add(offerLookup2);
        return offerLookups;
    }

    private OfferTransactionRequest getOfferTransactionRequest() {
        OfferTransactionRequest offerTransactionRequest = new OfferTransactionRequest();
        offerTransactionRequest.setHhId(54321L);
        offerTransactionRequest.setItemList(getItemList());
        return offerTransactionRequest;
    }

    private PEResponse getPeResponse() {
        PEResponse peResponse = new PEResponse();
        peResponse.setHhid(123L);
        peResponse.setDiscountedItemList(getItemList());
        return peResponse;
    }

    private EndOfSaleReq buildEndOfSaleReq() {
        OfferTransactionResponse offerTransactionResponse = new OfferTransactionResponse(HH_ID_FOR_END_OF_SALE, TRANSACTION_ID_FOR_END_OF_SALE, getItemList());
        EndOfSaleReq endOfSaleReq = new EndOfSaleReq(offerTransactionResponse, getOffers());
        return endOfSaleReq;
    }

    private List<Item> getItemList() {
        List<Item> itemList = new ArrayList<>();
        Item item1 = new Item("Coke", "12", 13.05);
        Item item2 = new Item("Thumpsup", "11", 14.05);
        itemList.add(item1);
        itemList.add(item2);
        return itemList;
    }
    private Mono<String> getMonoOfString() {
        return Mono.just("MONO_TO_STRING");
    }


    private OfferMeta getOfferMeta(){
        OfferMeta offerMetaDTO = new OfferMeta();
        offerMetaDTO.setDiscount("discount");
        offerMetaDTO.setEligibility("eligibility");
        offerMetaDTO.setExpiryTime(LocalDateTime.of(2021,01,01,01,01));
        offerMetaDTO.setStartTime(LocalDateTime.of(2020,01,01,01,01));
        offerMetaDTO.setOfferDesc("30 percent");
        offerMetaDTO.setOfferId(1L);
        offerMetaDTO.setOfferStatus("Active");
        offerMetaDTO.setRcptTxt("text");
        offerMetaDTO.setOfferType("aaa");
        return offerMetaDTO;
    }


    private List<OfferLookupDTO> getOfferLookupDto(){
        OfferLookupDTO offerLookupDTO = new OfferLookupDTO();
        offerLookupDTO.setId(123456L);
        offerLookupDTO.setIdType("1234");
        offerLookupDTO.setOfferId(1L);
        offerLookupDTO.setPreCondition("And");
        offerLookupDTO.setStoreId("001");
        offerLookupDTO.setTerminal("001");

        OfferMetaDTO offerMetaDTO = new OfferMetaDTO();
        offerMetaDTO.setDiscount("discount");
        offerMetaDTO.setEligibility("eligibility");
        offerMetaDTO.setExpiryTime(LocalDateTime.of(2021,01,01,01,01));
        offerMetaDTO.setStartTime(LocalDateTime.of(2020,01,01,01,01));
        offerMetaDTO.setOfferDesc("30 percent");
        offerMetaDTO.setOfferId(1L);
        offerMetaDTO.setOfferStatus("Active");
        offerMetaDTO.setRcptTxt("text");
        offerMetaDTO.setOfferType("aaa");

        offerLookupDTO.setOfferMetaDTO(offerMetaDTO);


        OfferLookupDTO offerLookupDTO1 = new OfferLookupDTO();
        offerLookupDTO1.setId(123456L);
        offerLookupDTO1.setIdType("1234");
        offerLookupDTO1.setOfferId(1L);
        offerLookupDTO1.setPreCondition("And");
        offerLookupDTO1.setStoreId("001");
        offerLookupDTO1.setTerminal("001");

        OfferMetaDTO offerMetaDTO1 = new OfferMetaDTO();
        offerMetaDTO1.setDiscount("discount");
        offerMetaDTO1.setEligibility("eligibility");
        offerMetaDTO1.setExpiryTime(LocalDateTime.of(2021,01,01,01,01));
        offerMetaDTO1.setStartTime(LocalDateTime.of(2020,01,01,01,01));
        offerMetaDTO1.setOfferDesc("20 percent");
        offerMetaDTO1.setOfferId(1L);
        offerMetaDTO1.setOfferStatus("Active");
        offerMetaDTO1.setRcptTxt("text");
        offerMetaDTO1.setOfferType("aaa");

        offerLookupDTO1.setOfferMetaDTO(offerMetaDTO);


        List<OfferLookupDTO> offerLookupDTOList = Arrays.asList(offerLookupDTO1, offerLookupDTO);
        return offerLookupDTOList;


    }
}